from datetime import datetime, timedelta

import pytest
from django.db.models import Q
from django.utils import timezone

from greg.models import (
    ScheduleTask,
    RoleType,
    Person,
    OrganizationalUnit,
    Role,
    Notification,
    Sponsor,
)
from greg.schedule import ExpiringRolesNotification


@pytest.fixture
def role_task():
    ScheduleTask.objects.create(
        name="role_expiration", last_completed=timezone.now() - timedelta(days=1)
    )


@pytest.fixture
def role(
    person: Person,
    role_type_bar: RoleType,
    unit_foo: OrganizationalUnit,
    sponsor_guy: Sponsor,
) -> Role:
    return Role.objects.create(
        person=person,
        type=role_type_bar,
        orgunit=unit_foo,
        start_date="2020-03-05",
        end_date=datetime.today() + timedelta(days=30),
        contact_person_unit="Contact Person",
        available_in_search=True,
        sponsor=sponsor_guy,
    )


@pytest.mark.django_db
def test_role_picked_up(role_task: ScheduleTask, role: Role):
    role_notification = ExpiringRolesNotification()
    assert len(Notification.objects.filter(~Q(operation="add"))) == 0
    role_notification.run()
    notification = Notification.objects.get(operation="expire_reminder")
    assert notification.identifier == role.id
    role_notification.run()
    notifications = Notification.objects.filter(operation="expire_reminder")
    assert len(notifications) == 1


@pytest.mark.django_db
def test_no_notification_for_role_not_about_to_expire(
    role_task: ScheduleTask, role: Role
):
    role.end_date = datetime.today() + timedelta(days=31)
    role.save()
    role_notification = ExpiringRolesNotification()
    assert len(Notification.objects.filter(operation="expire_reminder")) == 0
    # Role should not be picked up since it expires in 31 days
    role_notification.run()
    notifications = Notification.objects.filter(operation="expire_reminder")
    assert len(notifications) == 0
