import pytest

from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APIClient

from greg.models import InvitationLink, Person, Identity


@pytest.mark.django_db
def test_get_invite(client):
    """Forbid access with bad invitation link uuid"""
    response = client.post(reverse("gregui-v1:invite-verify"), data={"uuid": "baduuid"})
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_get_invite_ok(client, invitation_link):
    """Access okay with valid invitation link"""
    response = client.post(
        reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid}
    )
    assert response.status_code == status.HTTP_200_OK


@pytest.mark.django_db
def test_get_invite_expired(client, invitation_link_expired):
    """Forbid access with expired invite link"""
    response = client.post(
        reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link_expired.uuid}
    )
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_get_invited_info_no_session(client, invitation_link):
    response = client.get(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_get_invited_info_session_okay(
    client, invitation_link, person_foo_data, sponsor_guy_data, role_type_foo, unit_foo
):
    # get a session
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})
    # Get info
    response = client.get(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_200_OK
    data = response.json()
    assert data.get("person") == dict(
        **person_foo_data,
        email=None,
        mobile_phone=None,
        fnr=None,
        passport=None,
        feide_id=None,
    )
    assert data.get("sponsor") == dict(
        first_name=sponsor_guy_data["first_name"],
        last_name=sponsor_guy_data["last_name"],
    )
    assert data.get("role") == dict(
        start=None,
        end="2050-10-15",
        comments="",
        ou_name_en=unit_foo.name_en,
        ou_name_nb=unit_foo.name_nb,
        role_name_en=role_type_foo.name_en,
        role_name_nb=role_type_foo.name_nb,
    )


@pytest.mark.django_db
def test_get_invited_info_expired_link(
    client, invitation_link, invitation_expired_date
):
    # Get a session while link is valid
    client.get(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})
    # Set expire link to expire long ago
    invlink = InvitationLink.objects.get(uuid=invitation_link.uuid)
    invlink.expire = invitation_expired_date
    invlink.save()
    # Make a get request that should fail because invite expired after login, but
    # before get to userinfo
    response = client.get(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_invited_guest_can_post_information(
    client: APIClient, invitation_link, person_foo_data
):
    # get a session
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})

    person = invitation_link.invitation.role.person
    assert person.private_mobile is None

    # post updated info to confirm from guest
    new_email = "private@example.org"
    new_phone = "+4797543992"
    data = dict(person=dict(email=new_email, mobile_phone=new_phone))
    response = client.post(
        reverse("gregui-v1:invited-info"),
        data,
        format="json",
    )
    print(response.content)
    assert response.status_code == status.HTTP_200_OK

    # Check that the object was updated in the database
    assert Person.objects.count() == 1
    assert person.private_mobile.value == new_phone


@pytest.mark.django_db
def test_post_invited_info_expired_session(
    client, invitation_link, invitation_expired_date
):
    # get a session
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})
    # Set expire link to expire long ago
    invlink = InvitationLink.objects.get(uuid=invitation_link.uuid)
    invlink.expire = invitation_expired_date
    invlink.save()
    # post updated info to confirm from guest, should fail because of expired
    # invitation link
    response = client.post(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_post_invited_info_deleted_inv_link(client, invitation_link):
    # get a session
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})
    # Delete link
    invlink = InvitationLink.objects.get(uuid=invitation_link.uuid)
    invlink.delete()
    # post updated info to confirm from guest, should fail because of removed
    # invitation link
    response = client.post(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_post_invited_info_invalid_national_id_number(
    client, invitation_link, person_foo_data, person
):
    data = {
        "person": {
            "mobile_phone": "+4707543001",
            "email": "test@example.com",
            "fnr": "123",
        }
    }
    url = reverse("gregui-v1:invited-info")

    assert person.fnr is None

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    response = client.post(url, data, format="json")

    # The request should fail and the fnr-property should stay unchanged
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    person.refresh_from_db()
    assert person.fnr is None


@pytest.mark.django_db
def test_post_invited_info_valid_national_id_number(
    client, invitation_link, person_foo_data, person
):
    fnr = "11120618212"
    data = {
        "person": {
            "mobile_phone": "+4797543992",
            "email": "test@example.com",
            "fnr": fnr,
        }
    }
    url = reverse("gregui-v1:invited-info")

    assert person.fnr is None

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    response = client.post(url, data, format="json")

    assert response.status_code == status.HTTP_200_OK
    person.refresh_from_db()
    assert person.fnr.value == fnr


@pytest.mark.django_db
def test_email_update(client, invitation_link, person_foo_data, person):
    email_test = "test@example.com"
    url = reverse("gregui-v1:invited-info")

    Identity.objects.create(
        person=person,
        type=Identity.IdentityType.PRIVATE_EMAIL,
        value=email_test,
    )
    person.private_email.refresh_from_db()
    assert person.private_email.value == email_test

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    data = {"person": {"mobile_phone": "+4797543992", "email": "test2@example.com"}}
    response = client.post(url, data, format="json")
    assert response.status_code == status.HTTP_200_OK

    person.private_email.refresh_from_db()
    assert person.private_email.value == "test2@example.com"


@pytest.mark.django_db
def test_register_passport(client, invitation_link, person_foo_data, person):
    passport_information = "EN-123456789"
    data = {"person": {"mobile_phone": "+4797543992", "passport": passport_information}}
    url = reverse("gregui-v1:invited-info")

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    assert (
        Identity.objects.filter(
            person__id=person.id, type=Identity.IdentityType.PASSPORT_NUMBER
        ).count()
        == 0
    )

    response = client.post(url, data, format="json")

    assert response.status_code == status.HTTP_200_OK
    person.refresh_from_db()

    registered_passport = Identity.objects.filter(
        person__id=person.id, type=Identity.IdentityType.PASSPORT_NUMBER
    ).get()

    assert registered_passport.value == passport_information
